/** @file cocoa_volumepanel.mm

 HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
 Skelton for retropc emulator
 SDL edition + Cocoa GUI

 @author Sasaji
 @date   2015.04.30 -

 @brief [ volume panel ]
 */

#import "cocoa_gui.h"
#import "cocoa_volumepanel.h"
#import "../../config.h"
#import "../../emu.h"
#import "../../clocale.h"

extern EMU *emu;

@implementation CocoaVolumePanel
- (id)init
{
	const char *t[8] = {_("Master"),_("Beep"),_("6voice\nPSG"),_("9voice\nPSG"),_("Relay"),_("CMT"),_("FDD"),NULL};

	[super init];

	[self setPtr];

	self.title = [NSString stringWithUTF8String:_("Volume")];
	[self setShowsResizeIndicator:NO];

	NSRect re;
	CocoaView *view = [self contentView];
	int top = view.margin;
	int left = view.margin;
	int right = 0;

	for(int i=0; t[i] != NULL; i++) {
		if (i == 7) {
			top = re.origin.y + re.size.height + view.margin;
			left = view.margin;
			re = [view makeRect:left:top:re.origin.x + re.size.width - left:2];

			NSBox *sep = [[NSBox alloc] initWithFrame:re];
			[view addSubview:sep];

			top += view.margin;
		}
		re = [view makeRect:left:top:80:32];
		CocoaLabel *label = [CocoaLabel createWithoutFit:re title:t[i] align:NSCenterTextAlignment];
		[view addSubview:label];

		re = [view addHeight:re:80:128];
		CocoaSlider *slider = [CocoaSlider create:re index:i action:@selector(changeSlider:) value:[self volume:i]];
		[view addSubview:slider];

		re = [view addHeight:re:80:16];
		p_lbl[i] = [CocoaLabel createWithoutFit:re title:"00" align:NSCenterTextAlignment];
//		[p_lbl[i] setEditable:TRUE];
		[view addSubview:p_lbl[i]];
		[self setVolumeText:i];

		re = [view addHeight:re:80:32];
		CocoaCheckBox *check = [CocoaCheckBox create:re title:_("Mute") index:i action:@selector(changeMute:) value:[self mute:i]];
		[view addSubview:check];

		if (right < left) {
			right = left;
		}

		left += re.size.width;

		if (i == 0) {
			// separator
			re = [view makeRect:left:top:2:200];

			NSBox *sep = [[NSBox alloc] initWithFrame:re];

			[view addSubview:sep];

			left += 16;
		}
	}
	re.origin.x = right;

	re = [view addHeight:re:80:32];
	CocoaButton *btnClose = [CocoaButton create:re title:_("Close") action:@selector(dialogClose:)];
	[view addSubview:btnClose];

	[self adjustFrame];

	return self;
}

- (NSInteger)runModal
{
	return [NSApp runModalForWindow:self];
}

- (void)close
{
	[NSApp stopModalWithCode:NSOKButton];
	[super close];
}

- (void)dialogClose:(id)sender
{
	[self close];
}

- (void)setPtr
{
	int i = 0;
	p_volume[i++] = &config.volume;
	p_volume[i++] = &config.beep_volume;
	p_volume[i++] = &config.psg6_volume;
	p_volume[i++] = &config.psg9_volume;
	p_volume[i++] = &config.relay_volume;
	p_volume[i++] = &config.cmt_volume;
	p_volume[i++] = &config.fdd_volume;

	i = 0;
	p_mute[i++] = &config.mute;
	p_mute[i++] = &config.beep_mute;
	p_mute[i++] = &config.psg6_mute;
	p_mute[i++] = &config.psg9_mute;
	p_mute[i++] = &config.relay_mute;
	p_mute[i++] = &config.cmt_mute;
	p_mute[i++] = &config.fdd_mute;
}

- (void)setVolumeText:(int)idx
{
	char str[8];
	sprintf(str, "%02d", [self volume:idx]);
	[p_lbl[idx] setStringValue:[NSString stringWithUTF8String:str]];
}

- (bool)mute:(int)idx
{
	return *p_mute[idx];
}

- (int)volume:(int)idx
{
	return *p_volume[idx];
}

- (void)changeSlider:(CocoaSlider *)sender
{
	*p_volume[sender.index] = [sender intValue];
	[self setVolumeText:sender.index];
	emu->set_volume(0);
}

- (void)changeMute:(CocoaCheckBox *)sender
{
	*p_mute[sender.index] = ([sender state] == NSOnState);
	emu->set_volume(0);
}

@end
