/** @file sdl_common.h

	Skelton for retropc emulator

	@author Sasaji
	@date   2017.11.01 -

	@brief [ common header ]
*/

#ifndef _SDL_COMMON_H_
#define _SDL_COMMON_H_

#if defined(__MINGW32__) || _MSC_VER >= 1600
#include <stdint.h>
#else
#include <SDL.h>
#endif

// disable warnings C4189, C4995 and C4996 for microsoft visual c++ 2005
#if defined(_MSC_VER) && (_MSC_VER >= 1400)
#pragma warning( disable : 4819 )
#pragma warning( disable : 4995 )
#pragma warning( disable : 4996 )
#endif

// check memory leaks
#if defined(_DEBUG) && defined(_WIN32) && !defined(__MINGW32__)
//#define _CRTDBG_MAP_ALLOC
#undef _malloca
#include <stdlib.h>
#include <crtdbg.h>
#define malloc(s) _malloc_dbg(s, _NORMAL_BLOCK, __FILE__, __LINE__)
#if _MSC_VER > 1500
#define new new(_NORMAL_BLOCK, __FILE__, __LINE__)
#endif
#endif

// variable scope of 'for' loop for microsoft visual c++ 6.0 and embedded visual c++ 4.0
//#if defined(_MSC_VER) && (_MSC_VER == 1200)
//#define for if(0);else for
//#endif

#if defined(USE_GTK)
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>
#if defined(USE_OPENGL)
#if !GTK_CHECK_VERSION(3,16,0)
#undef USE_OPENGL
#endif
#endif /* USE_OPENGL */
#endif /* USE_GTK */

/********** platform dependency **********/
#if defined(_WIN32)
/********** windows **********/
#include <windows.h>
#include <tchar.h>

#ifdef __MINGW32__
#include <getopt.h>
#endif

#define USE_REC_VIDEO
#define USE_REC_VIDEO_VFW
//#define USE_REC_VIDEO_MMF		1
#define USE_REC_VIDEO_FFMPEG	1
#define USE_REC_VIDEO_WIN		1

#define USE_REC_AUDIO
#define USE_REC_AUDIO_WAVE
//#define USE_REC_AUDIO_MMF		2
#define USE_REC_AUDIO_FFMPEG	2

#elif (defined(__APPLE__) && defined(__MACH__))
/********** macosx **********/
#include <unistd.h>
#include "../../extra/tchar.h"
#define __stdcall

#define USE_REC_VIDEO
#define USE_REC_VIDEO_AVKIT		0x01
#define USE_REC_VIDEO_QTKIT		0x02
#define USE_REC_VIDEO_FFMPEG	1
#define USE_REC_VIDEO_COCOA		1

#define USE_REC_AUDIO
#define USE_REC_AUDIO_WAVE
#define USE_REC_AUDIO_AVKIT		0x10
#define USE_REC_AUDIO_FFMPEG	2

#else
/********** linux (Unix) **********/
#include <unistd.h>
#include "../../extra/tchar.h"
#define __stdcall

#define USE_LIB_GTK
#ifdef GUI_TYPE_AGAR
#define USE_X11_LEDBOX
#define USE_X11_VKEYBOARD
#else
#define USE_GTK_LEDBOX
#define USE_GTK_VKEYBOARD
#endif

#define USE_REC_VIDEO
#define USE_REC_VIDEO_FFMPEG	1
#define USE_REC_VIDEO_LIBPNG	1

#define USE_REC_AUDIO
#define USE_REC_AUDIO_WAVE
#define USE_REC_AUDIO_FFMPEG	2

#endif

#if defined(USE_SDL) || defined(USE_WX)
/* SDL1 */

#define CDelay(ms) SDL_Delay(ms)

# define SDL_Keycode		SDLKey

# define SDL_WINDOW_FULLSCREEN	SDL_FULLSCREEN
# define SDL_WINDOW_OPENGL		SDL_OPENGL

#elif defined(USE_SDL2)
/* SDL2 */

#define CDelay(ms) SDL_Delay(ms)

# if defined(GUI_TYPE_GTK_X11)
//#  define GUI_USE_FOREIGN_WINDOW
# endif

#endif

#endif
