/** @file sdl_csurface.cpp

	Skelton for retropc emulator

	@author Sasaji
	@date   2017.01.03 -

	@brief [ csurface ]
*/
#include "sdl_csurface.h"
#include "sdl_utils.h"

CSurface::CSurface()
{
	suf = NULL;
}
CSurface::CSurface(long width, long height, SDL_PixelFormat *pixel_format, int force_format)
{
	suf = NULL;

	this->Create(width, height, pixel_format, force_format);
}
CSurface::~CSurface()
{
	Release();
}

// Create surface
bool CSurface::Create(long width, long height, SDL_PixelFormat *pixel_format, int force_format)
{
	SDL_PixelFormat format;

	if (pixel_format) {
		format = *pixel_format;
	} else {
		memset(&format, 0, sizeof(format));
	}

	if (force_format) {
#if defined(_RGB555) || defined(_RGB565)
		format.BitsPerPixel = 16;
		format.BytesPerPixel = 2;
#elif defined(_RGB888)
		format.BitsPerPixel = 32;
		format.BytesPerPixel = 4;
#endif
	}

	switch(force_format) {
	case RGBA32:
		SDL_UTILS::set_pixel_format_for_opengl(&format);
		break;
	case BGRA32:
		SDL_UTILS::set_pixel_format_bgra(&format);
		break;
	case ARGB32:
		SDL_UTILS::set_pixel_format_argb(&format);
		break;
	case ABGR32:
		SDL_UTILS::set_pixel_format_rgba(&format);
		break;
	}

	suf = SDL_CreateRGBSurface(SDL_SWSURFACE, (int)width, (int)height
	  , format.BitsPerPixel, format.Rmask, format.Gmask, format.Bmask, format.Amask);

	return (suf != NULL);
}

bool CSurface::Create(VmRectWH *srcrect, SDL_PixelFormat *srcformat, int force_format)
{
	suf = SDL_UTILS::create_surface(srcrect, srcformat, force_format);
	return (suf != NULL);
}

bool CSurface::Create(VmRectWH *srcrect, CSurface &srcsurface, int force_format)
{
	suf = SDL_UTILS::create_surface(srcrect, srcsurface.Get(), force_format);
	return (suf != NULL);
}

bool CSurface::Create(CSurface &srcsurface, SDL_PixelFormat *pixel_format)
{
	suf = SDL_ConvertSurface(srcsurface.Get(), pixel_format, SDL_SWSURFACE);
	return (suf != NULL);
}

// release surface
void CSurface::Release()
{
	if (suf) {
		SDL_FreeSurface(suf);
		suf = NULL;
	}
}

SDL_Surface *CSurface::Get()
{
	return suf;
}

scrntype *CSurface::GetBuffer()
{
	return (scrntype *)suf->pixels;
}

int CSurface::GetBufferSize()
{
	return suf->w * suf->h * suf->format->BytesPerPixel;
}

bool CSurface::IsEnable()
{
	return (suf != NULL);
}

int CSurface::Width()
{
	return suf->w;
}

int CSurface::Height()
{
	return suf->h;
}

int CSurface::BitsPerPixel()
{
	return suf->format->BitsPerPixel;
}

int CSurface::BytesPerPixel()
{
	return suf->format->BytesPerPixel;
}

int CSurface::BytesPerLine()
{
	return suf->pitch;
}

SDL_PixelFormat *CSurface::GetPixelFormat()
{
	return suf->format;
}

bool CSurface::Lock()
{
	return (suf != NULL && SDL_LockSurface(suf) == 0);
}

void CSurface::Unlock()
{
	if (suf) SDL_UnlockSurface(suf);
}

int CSurface::Blit(SDL_Rect *src_re, SDL_Surface *dst, SDL_Rect *dst_re)
{
#if 0
	SDL_Rect sre, dre;
	if (src_re != NULL) {
		CP_RECT(*src_re, sre);
	} else {
		sre.x = 0; sre.y = 0;
		sre.w = suf->w; sre.h = suf->h;
	}
	if (dst_re != NULL) {
		CP_RECT(*dst_re, dre);
	} else {
		dre.x = 0; dre.y = 0;
		dre.w = dst->w; dre.h = dst->h;
	}
	SDL_UTILS::copy_surface(suf, &sre, dst, &dre); 
	return 0;
#else
	return SDL_BlitSurface(suf, src_re, dst, dst_re);
#endif
}

int CSurface::Blit(SDL_Rect *src_re, CSurface &dst, SDL_Rect *dst_re)
{
#if 0
	SDL_Rect sre, dre;
	if (src_re != NULL) {
		CP_RECT(*src_re, sre);
	} else {
		sre.x = 0; sre.y = 0;
		sre.w = suf->w; sre.h = suf->h;
	}
	if (dst_re != NULL) {
		CP_RECT(*dst_re, dre);
	} else {
		dre.x = 0; dre.y = 0;
		dre.w = dst.suf->w; dre.h = dst.suf->h;
	}
	SDL_UTILS::copy_surface(suf, &sre, dst.suf, &dre); 
	return 0;
#else
	return SDL_BlitSurface(suf, src_re, dst.suf, dst_re);
#endif
}
