/// @file parsewav.cpp
///
/// @brief wavef[^ <-> gf[^ <-> VAf[^@ϊNX
///
/// @author Sasaji
/// @date   2011.7.1
///
#define _USE_MATH_DEFINES 1
#include <math.h>
#include "parsewav.h"

//#define MAX_CARRIER_PATTERN	1
static const char	*carrier_pattern[4][4]={
	{
		"11001100",	// 0 (600 baud)
		"00110011",	// 0 (600 baud frip)
		"10101010",	// 1 (600 baud)
		"01010101",	// 1 (600 baud frip)
	},{
		"1100",		// 0 (1200 baud)
		"0011",		// 0 (1200 baud frip)
		"1010",		// 1 (1200 baud)
		"0101",		// 1 (1200 baud frip)
	},{
		"11",		// 0 (2400 baud)
		"00",		// 0 (2400 baud frip)
		"10",		// 1 (2400 baud)
		"01",		// 1 (2400 baud frip)
	},{
		"1100110011001100",	// 0 (300 baud)
		"0011001100110011",	// 0 (300 baud frip)
		"1010101010101010",	// 1 (300 baud)
		"0101010101010101",	// 1 (300 baud frip)
	}
};
#define IDX_PTN_600		0
#define IDX_PTN_1200	1
#define IDX_PTN_2400	2
#define IDX_PTN_300		3
// 2400{[̎̓GbWD
static const char *carrier_edge_pattern[]={
	"0010",		// 01 (2400 baud)
	"1101",		// 01 (2400 baud frip)
	"0100",		// 10 (2400 baud)
	"1011",		// 10 (2400 baud frip)
	NULL
};

//static const int c_baud_min[4]={ 3, 0, 1, 2 };

//static const int c_baud_ridx[4]={ 1, 2, 3, 0 };

//static const uint8_t cmt_ident[4] = { 0xff, 0x01, 0x3c, 0x00 };

/// @brief RXgN^
///
PARSEWAV::PARSEWAV()
{
	//
	wave_data = new Buf_t<uint8_t>(false);
	wave_correct_data = new Buf_t<uint8_t>(false);
	carrier_data = new Buf_t<char>(false, false, 36);

	err_num = 0;

	// wavt@Cwb_(o͗p)
	outwav.Init(48000, 8, 1);

	flog = NULL;
#ifdef _DEBUG
	if (param.debug_log) {
		flog = new FILEIO();
		flog->Fopen(_T("parsewav.log"), FILEIO_WRITE_BINARY);
	}
#endif
}

/// @brief fXgN^
///
PARSEWAV::~PARSEWAV()
{
	if (flog) {
		flog->Fclose();
		delete flog;
	}
	delete carrier_data;
	delete wave_correct_data;
	delete wave_data;
}

/// @brief G[bZ[Wo
///
/// @return G[bZ[Wւ̃|C^
///
///
///
_TCHAR* PARSEWAV::Errmsg()
{
	static _TCHAR str[1024];

	switch(err_num) {
		case 0:
			// no error
			_tcscpy(str,_T(""));
			break;
		case 1:
			_tcscpy(str,_T("File not found."));
			break;
		case 2:
			_tcscpy(str,_T("This is not the wav file which is formatted linear PCM."));
			break;
		case 3:
			_tcscpy(str,_T("This is not the frequency between 11025 and 48000Hz."));
			break;
		case 4:
			_tcscpy(str,_T("Cannot write to file."));
			break;
		default:
			_stprintf(str,_T("Unknown error: %d"),err_num);
			break;
	}
	return str;
}

/// @brief ̓t@C̃tH[}bg`FbN
///
/// @param[in] fp        ̓t@C
/// @param[in] file_type t@C 0:wavet@C 1:l3ct@C
/// @return    0: 1:t@CȂ 2..4:G[
int PARSEWAV::CheckFileFormat(FILEIO *fp, int file_type, int def_rate_num, int def_bits_num, int def_ch) {
	int rc = 0;

	// check wave header
	if (file_type == 0) {
		rc = conv.CheckWavFormat(fp, inwav, &sample_num);
		if (rc >= 0) {
			err_num = 0;
			sample_start = rc;
			sample_num_orig = sample_num;
			sample_pos = 0;
			sample_ms = 0;
		} else if (rc == -1) {
			// new file
			err_num = -rc;
			inwav.Init(get_sample_rate(def_rate_num), def_bits_num == 0 ? 8 : 16, def_ch);
			sample_start = 0;
			sample_num = 0;
			sample_pos = 0;
			sample_ms = 0;
		} else {
			err_num = -rc;
		}
	} else if (file_type == 1) {
		rc = check_l3c_format(fp);
	}

	return err_num;
}

/// @brief 
///
/// @param[in] fp        ̓t@C
/// @param[in] file_type t@C 0:wavet@C 1:l3ct@C
/// @param[in] w_onelen  TEho͗pobt@TCY
/// @return 0
int PARSEWAV::InitData(FILEIO *fp, int file_type, int baud_rate, int w_onelen)
{
	fio = fp;
	infile_type = file_type;

	wave_data->init();
	wave_correct_data->init();
	carrier_data->init();

//	wav_data = 0;
//	half_wave = param.GetHalfWave();

	// wavet@C̃wb_o
	outwav.Init(get_sample_rate(param.GetSampleRate()), param.GetSampleBits() == 0 ? 8 : 16, 1);

	conv.InitConvSampleData(DATA_BUFFER_SIZE);
	snd.InitConvSampleData(w_onelen);

	SetStartPhase(0);

	phase2 = 0;
	phase2_frip = 0;
	baud24_frip = 0;

	init_parameters();


	if (infile_type == 0) {
		infile_samples_onebit = inwav.GetSampleRate() / baud_rate;
		infile_samples_const  = inwav.GetSampleRate() / 600;
		dft.Init(st_lamda.samples[param.GetFskSpeed()], param.GetCorrectType());
	} else {
		infile_samples_onebit = (4800 << param.GetFskSpeed()) / baud_rate;
		infile_samples_const  = 8;
	}

	return 0;
}

///
///
///
///
///
void PARSEWAV::ClearData()
{
	wave_data->init();
	wave_correct_data->init();
	carrier_data->init();

	phase2 = 0;
	phase2_frip = 0;
	baud24_frip = 0;
}

///
///
///
///
///
void PARSEWAV::init_parameters()
{
	st_pa_carr.x0_prev = 0.0;
	st_pa_carr.sample_cnt = 0;
	st_pa_carr.wav_prev = 0;
	st_pa_carr.carr_prev = 0;
	st_pa_carr.odd = 0;

	st_lamda.us_delta = 1000000.0 / (double)inwav.GetSampleRate();
	for(int n=0; n<3; n++) {
		st_lamda.samples[n] = (double)inwav.GetSampleRate() / (double)(1200 << n);

		st_lamda.us_range[n] = 1000000.0 / (double)(1200 << n) / (double)(1 << param.GetHalfWave());
		st_lamda.us[n] = 1000000.0 / (double)param.GetFreq(n) / (double)(1 << param.GetHalfWave());
		st_lamda.us_avg[n] = st_lamda.us[n];
	}
	for(int n=0; n<2; n++) {
		st_lamda.us_mid[n] = 1000000.0 / (double)(1800 << n) / (double)(1 << param.GetHalfWave());
		st_lamda.us_mid_avg[n] = st_lamda.us_mid[n];

		st_lamda.us_limit[n] = 1000000.0 / (double)(1200 << (n + 3));
	}
	int ns = param.GetFskSpeed();
	for(int n=0; n<2; n++, ns++) {
		st_lamda.us_min[ns] = st_lamda.us[ns] - st_lamda.us_range[ns] * param.GetRange(n) / 100.0;
		st_lamda.us_max[ns] = st_lamda.us[ns] + st_lamda.us_range[ns] * param.GetRange(n) / 100.0;
	}
}

///
///
///
///
///
void PARSEWAV::SetBaudAndFrequency(int baud_rate, int spd)
{
	param.SetBaud(spd & 3);
	param.SetFskSpeed((spd & 4) >> 2);

	infile_samples_onebit = inwav.GetSampleRate() / baud_rate;

	init_parameters();

	if (infile_type == 0) {
		dft.Init(st_lamda.samples[param.GetFskSpeed()], param.GetCorrectType());
	}
}

///
///
///
///
///
void PARSEWAV::SetParseParamerers(int half, int reverse, int correct, int corr_type)
{
	param.SetHalfWave(half);
	param.SetReverseWave(reverse);
	param.SetCorrectWave(correct);
	param.SetCorrectType(corr_type);
}

///
///
/// @param[in] dir 0:Đ 1: -1:߂
///
///
void PARSEWAV::SetStartPhase(int dir)
{
	switch(dir) {
	case 1:
		if (infile_type == 1) {
			// .l3c
			phase1 = 14;
			start_phase = 14;
		} else {
			// .wav
			phase1 = 4;
			start_phase = 4;
		}
		break;
	case -1:
		if (infile_type == 1) {
			// .l3c
			phase1 = 18;
			start_phase = 18;
		} else {
			// .wav
			phase1 = 8;
			start_phase = 8;
		}
		break;
	default:
		if (infile_type == 1) {
			// .l3c
			phase1 = 11;
			start_phase = 11;
		} else {
			// .wav
			phase1 = 0;
			start_phase = 0;
		}
	}
}

/// @brief wavet@C̃wb_obt@ɏo͂
///
/// @param[out] buffer obt@
/// @return wb_
///
int PARSEWAV::InitFileHeader(uint8_t *buffer)
{
	return outwav.Out(buffer);
}

/// @brief wavet@CɃt@CTCY
///
/// @param[in] fp ̓t@C
///
///
void PARSEWAV::SetFileHeader(FILEIO *fp)
{
	PARSEWAV_FORMAT::OutSize(fp);
}

/// @brief Tv[gԂ
///
/// @param[in] num 0:11025Hz 1:22050Hz 2:44100Hz
/// @return Tv[g
int PARSEWAV::get_sample_rate(int num) {
	int rate = 48000;

	switch(num) {
		case 0:
			rate = 11025;
			break;
		case 1:
			rate = 22050;
			break;
		case 2:
			rate = 44100;
			break;
	}
	return rate;
}

/// @brief l3ct@C̃tH[}bg`FbNi`FbNĂȂj
///
/// @param[in] fp ̓t@C
/// @return 0
///
int PARSEWAV::check_l3c_format(FILEIO *fp)
{
	fp->Fseek(0, FILEIO_SEEK_END);
	sample_num = fp->Ftell();
	fp->Fseek(0, FILEIO_SEEK_SET);

	sample_start = 0;
	sample_pos = 0;

	return 0;
}

/// @brief wavt@CPTvǂ
///
/// @param[in] blk_size 1: uint8_tŕԂ  2: int16_tŕԂ
/// @param[in] reverse 1: g`𔽓]
/// @return ϊPTvf[^ or EOF
///
int PARSEWAV::get_wav_sample(int blk_size, int reverse)
{
	int l = 0;
	int h = 0;
	int in_bits = inwav.GetSampleBits();

	// 1Tv𓾂
	l = fio->Fgetc();
	if (l == EOF) {
		return l;
	}
	if(in_bits == 16) {
		// int16_t̏ꍇ
		h = fio->Fgetc();
		if (h >= 128) {
			h = h - 256;
		}
		if (reverse) {
			h *= -1;
			if (h >= 128) h = 127;
			l *= -1;
		}
	} else {
		if (reverse) {
			l = 256 - l;
			if (l >= 256) l = 255;
		}
	}
	// XeȈꍇA𖳎
	for(int i = 2; i <= inwav.GetChannels(); i++) {
		fio->Fgetc();
		if(in_bits == 16) {
			fio->Fgetc();
		}
	}
	sample_pos++;
	sample_ms = (uint32_t)(1000.0 * (double)sample_pos / (double)inwav.GetSampleRate());

	if (blk_size == 1) {
		if(in_bits == 16) {
			// int16_t -> uint8_t
			return ((h + 128) & 0xff);
		} else {
			return (l & 0xff);
		}
	} else {
		if(in_bits == 16) {
			return h * 256 + l;
		} else {
			// uint8_t -> int16_t
			return (l - 128) * 256;
		}
	}
}

/// @brief wavt@CPTvXLbv
///
/// @param[in] dir
/// @return XLbv
///
int PARSEWAV::skip_wav_sample(int dir)
{
	int in_bits = inwav.GetSampleBits();
	int offset = 0;
//	if (sample_pos < 100) {
//		int n= 0;
//	}

	if ((int)sample_pos + dir <= 0) {
		dir = (int)sample_pos * -1;
	} else if (sample_pos + (size_t)dir >= sample_num) {
		dir = (int)(sample_num - sample_pos);
	}
	offset = dir * inwav.GetChannels() * in_bits / 8;

	fio->Fseek(offset, FILEIO_SEEK_CUR);

	sample_pos+=dir;
	sample_ms = (uint32_t)(1000.0 * (double)sample_pos / (double)inwav.GetSampleRate());

	return dir;
}

/// @brief wavt@CPTvǂ(int16_tŕԂ)
///
/// @param[in] reverse 1: g`𔽓]
/// @return ϊPTvf[^
//int PARSEWAV::get_wav_sample(int reverse)
//{
//	return get_wav_sample(2, reverse);
//}

/// @brief wavt@CPTvǂŃobt@ɒǋL
///
/// @param[in,out] w_data Tvf[^p̃obt@(ǋLĂ)
/// @param[in] size ǂݍރTCY
/// @param[in] reverse 1: g`𔽓]
/// @return ǂݍ񂾃f[^̒
///
size_t PARSEWAV::get_wav_sample(Buf_t<uint8_t> *w_data, int size, int reverse)
{
	int l;

	while(w_data->isFull(0) != true && size > 0 && sample_pos < sample_num) {
		l = get_wav_sample(1, reverse);

		w_data->add((uint8_t)l);
		size--;
	}
	if (sample_pos >= sample_num) {
		w_data->last = true;
	}
	return w_data->w_pos;
}

/// @brief l3ct@C(grbgf[^)PTvXLbv
///
/// @param[in] dir
/// @return XLbv
///
int PARSEWAV::skip_l3c_sample(int dir)
{
	if ((int)sample_pos + dir <= 0) {
		dir = (int)sample_pos * -1;
	} else if (sample_pos + (size_t)dir >= sample_num) {
		dir = (int)(sample_num - sample_pos);
	}
	fio->Fseek(dir, FILEIO_SEEK_CUR);

	sample_pos+=dir;
	sample_ms = (uint32_t)(1000.0 * (double)sample_pos / 4800.0);

	return dir;
}

/// @brief l3ct@C(grbgf[^)Pf[^ǂŃobt@ɒǋL
///
/// @param[in,out] c_data gf[^p̃obt@(ǋLĂ)
/// @param[in] size ǂݍރTCY
/// @return ǂݍ񂾃f[^̒
///
///
size_t PARSEWAV::get_l3c_sample(Buf_t<char> *c_data, int size)
{
	int l;

	while(c_data->isFull(0) != true && size > 0 && sample_pos < sample_num) {
		l = fio->Fgetc();
		l &= 0xff;

		sample_pos++;
		sample_ms = (uint32_t)(1000.0 * (double)sample_pos / 4800.0);
		if (l == '\r' || l == '\n') {
			continue;
		}
		c_data->add(l, sample_ms);
		size--;
	}
	if (sample_pos >= sample_num) {
		c_data->last = true;
	}
	return c_data->w_pos;
}

/// @brief 2400Hz1200Hz̔gărbgf[^ɕϊ
///
/// @param[in]   fsk_spd   1:{FSK
/// @param[in]   w_data    WAVTvf[^
/// @param[out]  c_data  "10": 2400Hz "1100":1200Hz "?HHLL":Too long
/// @return WAV1Tvf[^
int PARSEWAV::decode_to_carrier_wave_bn(int fsk_spd, Buf_t<uint8_t> *w_data, Buf_t<char> *c_data)
{
	char bit_data[10];

	int data = 0;
	uint8_t  dat;
	uint32_t ms = 0;
	bool hi = false;
	bool found = false;

	double x0 = 0.0;
	double lamda = 0.0;

	// find next trigger point
	while(w_data->isEOB(0) != true) {
		w_data->get(dat, ms);
		data = (int)dat - 128;

		if (st_pa_carr.wav_prev < 0 && data >= 0) {
			// l -> h ɂȂʒug(us)߂
			hi = true;
			x0 = - (st_pa_carr.wav_prev * st_lamda.us_delta / (data - st_pa_carr.wav_prev));
			lamda = st_pa_carr.x0_prev + (st_pa_carr.sample_cnt * st_lamda.us_delta) + x0;
			// gZꍇ̓mCYƔfďp
			if (lamda >= st_lamda.us_limit[fsk_spd]) {
				st_pa_carr.x0_prev = st_lamda.us_delta - x0;
				st_pa_carr.odd = 1 - st_pa_carr.odd;
				found = true;
				break;
			}
		} else if (st_pa_carr.wav_prev > 0 && data <= 0 && param.GetHalfWave() == 1) {
			// h -> l ɂȂʒug(us)߂
			hi = false;
			x0 = - (st_pa_carr.wav_prev * st_lamda.us_delta / (data - st_pa_carr.wav_prev));
			lamda = st_pa_carr.x0_prev + (st_pa_carr.sample_cnt * st_lamda.us_delta) + x0;
			// gZꍇ̓mCYƔfďp
			if (lamda >= st_lamda.us_limit[fsk_spd]) {
				st_pa_carr.x0_prev = st_lamda.us_delta - x0;
				st_pa_carr.odd = 1 - st_pa_carr.odd;
				found = true;
				break;
			}
		}
		st_pa_carr.sample_cnt++;
		st_pa_carr.wav_prev = data;
	}
	if (found != true) {
		return data;
	}

	if (param.GetHalfWave() == 0) {
		// Sg
		if (st_lamda.us_min[fsk_spd] <= lamda && lamda <= st_lamda.us_max[fsk_spd]) {
			// long (half wave)
			strcpy(bit_data,"1100");
			st_pa_carr.carr_prev = 4;
			st_lamda.us_avg[fsk_spd] = (st_lamda.us_avg[fsk_spd] + lamda) / 2.0;

			rep_p1_sample_num[0]++;
		}
		else if (st_lamda.us_min[fsk_spd+1] <= lamda && lamda <= st_lamda.us_max[fsk_spd+1]) {
			// short (half wave)
			strcpy(bit_data,"10");
			st_pa_carr.carr_prev = 2;
			st_lamda.us_avg[fsk_spd+1] = (st_lamda.us_avg[fsk_spd+1] + lamda) / 2.0;

			rep_p1_sample_num[1]++;
		}
		else if (st_lamda.us_mid[fsk_spd] <= lamda && lamda < st_lamda.us_min[fsk_spd]) {
			// middle long
			strcpy(bit_data,"?1100");
			st_pa_carr.carr_prev = 4;
			st_lamda.us_mid_avg[fsk_spd] = (st_lamda.us_mid_avg[fsk_spd] + lamda) / 2.0;

			rep_p1_sample_num[2]++;
		}
		else if (st_lamda.us_max[fsk_spd+1] < lamda && lamda < st_lamda.us_mid[fsk_spd]) {
			// middle short
			strcpy(bit_data,"?10");
			st_pa_carr.carr_prev = 2;
			st_lamda.us_mid_avg[fsk_spd] = (st_lamda.us_mid_avg[fsk_spd] + lamda) / 2.0;

			rep_p1_sample_num[2]++;
		}
		else if (st_lamda.us_max[fsk_spd] < lamda) {
			// too long
			strcpy(bit_data,"?HHLL");
			st_pa_carr.carr_prev = 5;
			rep_p1_sample_num[3]++;
		}
		else if (lamda < st_lamda.us_min[fsk_spd+1]) {
			// too short
			strcpy(bit_data,"?HL");
			st_pa_carr.carr_prev = 1;
			rep_p1_sample_num[4]++;
		}
		else {
			// error
			strcpy(bit_data,"??");
			st_pa_carr.carr_prev = 0;
			rep_p1_sample_num[5]++;
		}
	} else {
		// g
		if (st_lamda.us_min[fsk_spd] <= lamda && lamda <= st_lamda.us_max[fsk_spd]) {
			// long (half wave)
			if (hi) {
				strcpy(bit_data,"00");
				if (st_pa_carr.carr_prev != 4) st_pa_carr.odd = 1;
				st_pa_carr.carr_prev = -4;
			} else {
				strcpy(bit_data,"11");
				if (st_pa_carr.carr_prev != -4) st_pa_carr.odd = 1;
				st_pa_carr.carr_prev = 4;
			}
			st_lamda.us_avg[fsk_spd] = (st_lamda.us_avg[fsk_spd] + lamda) / 2.0;

			rep_p1_sample_num[0]++;
		}
		else if (st_lamda.us_min[fsk_spd+1] <= lamda && lamda <= st_lamda.us_max[fsk_spd+1]) {
			// short (half wave)
			if (hi) {
				strcpy(bit_data,"0");
				if (st_pa_carr.carr_prev != 2) st_pa_carr.odd = 1;
				st_pa_carr.carr_prev = -2;
			} else {
				strcpy(bit_data,"1");
				if (st_pa_carr.carr_prev != -2) st_pa_carr.odd = 1;
				st_pa_carr.carr_prev = 2;
			}
			st_lamda.us_avg[fsk_spd+1] = (st_lamda.us_avg[fsk_spd+1] + lamda) / 2.0;

			rep_p1_sample_num[1]++;
		}
		else if (st_lamda.us_max[fsk_spd+1] < lamda && lamda < st_lamda.us_min[fsk_spd]) {
			// middle
			if (hi) {
				if ((st_pa_carr.odd == 0 && st_pa_carr.carr_prev == 4)
				 || (st_pa_carr.odd == 1 && st_pa_carr.carr_prev == 2)) {
					strcpy(bit_data,"?00");
					st_pa_carr.carr_prev = -4;
				} else {
					strcpy(bit_data,"?0");
					st_pa_carr.carr_prev = -2;
				}
			} else {
				if ((st_pa_carr.odd == 0 && st_pa_carr.carr_prev == -4)
				 || (st_pa_carr.odd == 1 && st_pa_carr.carr_prev == -2)) {
					strcpy(bit_data,"?11");
					st_pa_carr.carr_prev = 4;
				} else {
					strcpy(bit_data,"?1");
					st_pa_carr.carr_prev = 2;
				}
			}
			st_lamda.us_mid_avg[fsk_spd] = (st_lamda.us_mid_avg[fsk_spd] + lamda) / 2.0;

			rep_p1_sample_num[2]++;
		}
		else if (st_lamda.us_max[fsk_spd] < lamda) {
			// too long
			if (hi) {
				strcpy(bit_data,"?LL");
				st_pa_carr.carr_prev = -5;
			} else {
				strcpy(bit_data,"?HH");
				st_pa_carr.carr_prev = 5;
			}
			rep_p1_sample_num[3]++;
		}
		else if (lamda < st_lamda.us_min[fsk_spd+1]) {
			// too short
			if (hi) {
				strcpy(bit_data,"?L");
				st_pa_carr.carr_prev = -1;
			} else {
				strcpy(bit_data,"?H");
				st_pa_carr.carr_prev = 1;
			}
			rep_p1_sample_num[4]++;
		}
		else {
			// error
			strcpy(bit_data,"??");
			st_pa_carr.carr_prev = 0;

			rep_p1_sample_num[5]++;
		}
	}
	// f[^ǉ
	int llen = 0;
	if (bit_data[0] != '?') {
		// OK data
		llen = c_data->adds(bit_data, ms);
	} else {
		// NG data
		llen = c_data->adds(&bit_data[1], ms);
#if 0
		if (param.debug_log > 1) {
			// fobOO
			sprintf(cbuff, "p1 w:%12d(%s) c:%12d: %s\n"
				, w_data->total_r_pos
				, get_time_cstr(ms)
				, c_data->total_w_pos
				, bit_data);
				if (fiolog) fwrite((void *)cbuff, strlen(cbuff), 1, fiolog);
		}
#endif
	}
//	if (flog) {
//		flog->Fprintf("c_data: w:%d r:%d l:%d %s\n", c_data->w_pos, c_data->r_pos, llen, bit_data);
//	}

	st_pa_carr.wav_prev = data;
	st_pa_carr.sample_cnt = 0;

	return data;
}

/// @brief vp^[T
///
/// @param[in]   c_data g
/// @param[out]  s_data VAf[^
/// @param[in]   baud   600baud:0 1200baud:1 2400baud:2 300baud:3
/// @param[out]  frip   1->0->1->0: 0 0->1->0->1:1
/// @param[out]  c_onedata NULL or VAf[^̔gf[^
/// @return X^[gʒu / -1 Ȃ
int PARSEWAV::find_start_carrier_bit(Buf_t<char> *c_data, char *s_data, int baud, int *frip, char *c_onedata, int *c_onelen)
{
	int pos1[4];
	int pos = -1;
	int idx_ptn = (baud & 3);
	int best_num = -1;
	int best_pos = (int)(c_data->size);
	int i;
//	int ptn;
    char bit[4];
//	uint32_t ms;

	strcpy(bit, "?");

	// 2400{[̎̓GbWT
	if (idx_ptn == IDX_PTN_2400) {
		for(i=0; i<4; i++) {
			pos1[i]=c_data->find(0, carrier_edge_pattern[i]);
		}
		for(i=0; i<4; i++) {
			if (pos1[i] >= 0 && best_pos > pos1[i]) {
				best_pos = pos1[i];
				best_num = i;
			}
		}
		if (best_num >= 0) {
			// 
			pos = best_pos + (int)strlen(carrier_edge_pattern[best_num]);

			if ((best_num % 2) == 0) {
				(*frip) = 0;
			} else {
				(*frip) = 1;
			}
			if ((best_num / 2) == 0) {
				strcpy(bit, "01");
			} else {
				strcpy(bit, "10");
			}
		}
	}

	// vp^[T
	if (best_num < 0) {
		for(i=0; i<4; i++) {
			pos1[i]=c_data->find(0, carrier_pattern[idx_ptn][i]);
		}
		for(i=0; i<4; i++) {
			if (pos1[i] >= 0 && best_pos > pos1[i]) {
				best_pos = pos1[i];
				best_num = i;
			}
		}

		if (best_num >= 0) {
			pos = best_pos + (int)strlen(carrier_pattern[idx_ptn][best_num]);

			if ((best_num % 2) == 0) {
				(*frip) = 0;
			} else {
				(*frip) = 1;
			}
			if ((best_num / 2) == 0) {
				strcpy(bit, "0");
			} else {
				strcpy(bit, "1");
			}
		}
	}
	if (pos >= 0) {
		// f[^L
#if 0
		ms = c_data->get_ms(best_pos);

		if (param.debug_log > 1) {
			// fobOO
			sprintf(cbuff, "p2 fst c:%12d(%s) w_pos:%d pos:%d frip:%d ("
				, c_data->total_r_pos
				, get_time_cstr(ms)
				, s_data->total_w_pos
				, best_pos, *frip);
			strncat(cbuff, c_data->curptr(best_pos), pos - best_pos);
			strcat(cbuff, ")\n");
				if (fiolog) fwrite((void *)cbuff, strlen(cbuff), 1, fiolog);
		}
#endif
		if (c_onedata != NULL) {
			*c_onelen = pos;
			strncat(c_onedata, &c_data->data[c_data->r_pos], *c_onelen);
		}
		c_data->r_pos += pos;
		c_data->total_r_pos += pos;
	} else {
		// f[^Ȃ
		if (c_onedata != NULL) {
			*c_onelen = c_data->w_pos - c_data->r_pos;
			strncat(c_onedata, &c_data->data[c_data->r_pos], *c_onelen);
		}
		c_data->total_r_pos += c_data->w_pos - c_data->r_pos;
		c_data->r_pos = c_data->w_pos;
	}

	strcpy(s_data, bit);

	return pos;
}

/// @brief g(2400/1200Hz)̃f[^VAf[^Prbgo
///
/// @param[in]  c_data  g
/// @param[out] s_data    VAf[^
/// @param[in]  baud    600baud:0 1200baud:1 2400baud:2 300baud:3
/// @param[in,out]  frip    1->0->1->0: 0 0->1->0->1:1
/// @param[out] c_onedata NULL or VAf[^̔gf[^
/// @return ̈ʒu
int PARSEWAV::decode_to_serial_data(Buf_t<char> *c_data, char *s_data, int baud, int *frip, char *c_onedata, int *c_onelen)
{
	int pos = -1;
	int idx_ptn = (baud & 3);
	int len;
//	int ptn;
	char bit_data[4];
	char *pptn;

	strcpy(bit_data, "");

	len = (int)strlen(carrier_pattern[idx_ptn][*frip]);
	pptn = (char *)carrier_pattern[idx_ptn][*frip];
	if (c_data->cmp(0, pptn, len) == 0) {	// 0
		// 0
		strcpy(bit_data, "0");
		pos = len;
		if (idx_ptn == IDX_PTN_2400) {
			// 2400{[0̏ꍇAfrip
			*frip = (1 - *frip);
		}
	}
	if (pos < 0) {
		len = (int)strlen(carrier_pattern[idx_ptn][2 + *frip]);
		pptn = (char *)carrier_pattern[idx_ptn][2 + *frip];
		if (c_data->cmp(0, pptn, len) == 0)	{ // 1
			// 1
			strcpy(bit_data, "1");
			pos = len;
		}
	}

//	ms = c_data->get_ms(0);

	if (pos >= 0) {
		// f[^ǉ
		if (c_onedata != NULL) {
			*c_onelen = pos;
			strncat(c_onedata, &c_data->data[c_data->r_pos], pos);
		}
		c_data->r_pos += pos;
		c_data->total_r_pos += pos;
	} else {
		// vp^[Ȃ
		strcpy(bit_data, "?");

		if (c_onedata != NULL) {
			*c_onelen = 1;
			strncat(c_onedata, &c_data->data[c_data->r_pos], 1);
		}
		if (c_data->sublen() > 0) {
			c_data->r_pos += 1;
			c_data->total_r_pos += 1;
		}
#if 0
		rep_p2_error_num++;

		if (param.debug_log > 1) {
			// fobOO
			sprintf(cbuff,"p2 c2s c:%12d(%s) s:%8d: error ("
				, c_data->total_r_pos
				, get_time_cstr(ms)
				, s_data->total_w_pos
				);
			strncat(cbuff, c_data->curptr(0)
				, c_data->sublen() >= len ? len : c_data->sublen());
			strcat((char *)cbuff, ")\n");
				if (fiolog) fwrite((void *)cbuff, strlen(cbuff), 1, fiolog);
		}
#endif
	}

	strcpy(s_data, bit_data);

	return pos;
}

/// @brief waveTvf[^g(1200/2400Hz)͂
/// @brief rbgf[^ɕϊB(FSK)
///
/// @param[in]  fsk_spd 1:{FSK
/// @param[in]  start_phase JntF[Y
/// @param[in]  correct_type ␳@(gp)
/// @param[in]  reverse g`]
/// @param[out] w_onedata waveobt@
/// @param[in,out] w_onelen waveobt@/񂾒
/// @return     f[^I true
bool PARSEWAV::decode_phase1(int fsk_spd, int start_phase, int corret_type, int reverse, char *w_onedata, int *w_onelen)
{
	bool last = false;
	Buf_t<uint8_t> *wn_data;

	if (infile_type == 0 && param.GetCorrectWave() == 1) {
		wn_data = wave_correct_data;
	} else {
		wn_data = wave_data;
	}

	while(phase1 >= 0 && last == false) {
		switch(phase1) {
			case 0:
				// WAVt@Cǂݍ
				inwav_dirpos = (int)get_wav_sample(wave_data, infile_samples_onebit, reverse);
				phase1 = 1;
				break;
			case 1:
				// WAVTvf[^␳
				if (infile_type == 0 && param.GetCorrectWave() == 1) {
					dft.Calcrate(wave_data, wave_correct_data);
				}
				// wavt@Cϊ
				if (w_onedata) {
					*w_onelen = (int)snd.OutConvSampleData(wn_data->startptr(), inwav.GetSampleRate(), 8, 1, wn_data->length()
						,(uint8_t *)w_onedata, sndwav.GetSampleRate(), sndwav.GetSampleBits(), *w_onelen, 0);
				}
				// f[^Ō܂œǂ񂾏ꍇ
				if (param.GetCorrectWave() == 1) {
					if (wave_data->last && wave_data->isEOB((int)st_lamda.samples[0])) {
						wave_correct_data->last = true;
					}
				}
				phase1 = 2;
				break;
			case 2:
				// L3C֕ϊ
				if (carrier_data->isFull(1) != true) {
					decode_to_carrier_wave_bn(fsk_spd, wn_data, carrier_data);
				}
				if (carrier_data->isFull(8)) {
					// cobt@ςɂȂfo
					phase1 = 21;
				} else if (wn_data->last != true && wn_data->isEOB((int)st_lamda.samples[0])) {
					// cobt@ςɂȂ܂ŌJԂ
					phase1 = 21;
				} else if (wn_data->last && wn_data->isEOB(0)) {
					// Ō̃f[^
					carrier_data->last = true;
					phase1 = -1;
				}
				break;
			case 4:
				// F.F.
				inwav_dirpos = skip_wav_sample(infile_samples_const);
				if (w_onedata) {
					*w_onelen = 0;
				}

				phase1 = start_phase;
				last = true;
				break;
			case 8:
				// Rewind
				inwav_dirpos = - skip_wav_sample(-infile_samples_const);
				if (w_onedata) {
					*w_onelen = 0;
				}

				phase1 = start_phase;
				last = true;
				break;
			case 11:
				// L3Ct@Cǂݍ
				inwav_dirpos = (int)get_l3c_sample(carrier_data, infile_samples_onebit);

				phase1 = start_phase;
				last = true;
				break;
			case 14:
				// F.F.
				inwav_dirpos = skip_l3c_sample(infile_samples_const);

				phase1 = start_phase;
				last = true;
				break;
			case 18:
				// Rewind
				inwav_dirpos = - skip_l3c_sample(-infile_samples_const);

				phase1 = start_phase;
				last = true;
				break;
			case 21:
				// cobt@̍ŏɃRs[̃^[ŉ͂邽
				wave_data->shift();
				wave_correct_data->shift();

				phase1 = start_phase;
				last = true;
				break;
		}
	}

	return carrier_data->last;
}

/// @brief g(1200/2400Hz)f[^VAf[^ɕϊ
/// Ń{[[gɂăVAf[^ςĂ
/// @param[out] s_data VAf[^
/// @param[out] c_onedata NULL or VAf[^1rbg̔g
/// @return tF[Yԍ
int PARSEWAV::decode_phase2(char *s_data, char *c_onedata, int *c_onelen)
{
	int step = 0;
//	bool break_data = false;
	int baud = param.GetBaud();

	switch (phase2) {
		case 0:
			step = find_start_carrier_bit(carrier_data, s_data, baud, &phase2_frip, c_onedata, c_onelen);
			if (carrier_data->last && carrier_data->isEOB(0)) {
				phase2 = -1;
				break;
			}
			if (step >= 0) {
				phase2 = 1;
			}
//			break_data = true;
			break;
		case 1:
			step = decode_to_serial_data(carrier_data, s_data, baud, &phase2_frip, c_onedata, c_onelen);
			if (carrier_data->last && carrier_data->isEOB(0)) {
				phase2 = -1;
				break;
			}
			if (step < 0) {
				phase2 = 0;
			}
//			break_data = true;
			break;
	}
	if (sample_pos >= sample_num) {
		phase2 = -1;
	}

//	if (flog) {
//		if (carrier_data->w_pos < carrier_data->r_pos) {
//			flog->Fprintf("phase2:%d c_w:%d c_r:%d\n",phase2, c_data->w_pos, c_data->r_pos);
//		}
//	}

	// obt@̖ɋ߂Â
	// c̖̓f[^obt@̐擪ɃRs[
	// ̃^[ŉ͂sB
	if (carrier_data->isEOB(8)) {
		carrier_data->shift();
	}

	return phase2;
}

/// @brief l3b -> l3c VAf[^grbgf[^ɕϊ
///
/// @param[in] data VAf[^i1rbgj
/// @param[out] buffer gf[^
/// @param[out] c_len  gf[^̒
/// @return bufferɏ񂾃TCY
int PARSEWAV::encode_phase2(uint8_t data, uint8_t *buffer, int *c_len)
{
	int idx_ptn = (param.GetBaud() & 3);
	int len = 0;

	if (data & 0x01) {
		// 1
		len = (int)strlen(carrier_pattern[idx_ptn][2 + baud24_frip]);
		strncpy((char *)buffer, carrier_pattern[idx_ptn][2 + baud24_frip], len);
	} else {
		// 0
		len = (int)strlen(carrier_pattern[idx_ptn][baud24_frip]);
		strncpy((char *)buffer, carrier_pattern[idx_ptn][baud24_frip], len);
		if (idx_ptn == IDX_PTN_2400) {
			baud24_frip = (1-baud24_frip);
		}
	}
	if (c_len != NULL) {
		*c_len = len;
	}
	return len;
}

/// @brief VAf[^grbgf[^ɕϊwaveTvf[^ɕϊ
///
/// @param[in] data VAf[^i1rbgj
/// @param[out] buffer wavef[^
/// @param[out] c_len  gf[^̒
/// @return bufferɏ񂾃TCY
int PARSEWAV::encode_phase1(uint8_t data, uint8_t *buffer, int *c_len)
{
	int len = 0;
	int pos = 0;
	int mag = (param.GetFskSpeed() ? 5 : 10);
	uint8_t wav_samples[10];

	len = encode_phase2(data, (uint8_t *)carrier_data->data, c_len);

	// convert to wave sample data
	memset(wav_samples, 0xf0, 10);
	wave_data->clear();

	for(int j=0; j < len; j++) {
		if (j > 0 && (carrier_data->data[j-1] & 1) == (carrier_data->data[j] & 1)) {
			wav_samples[0] = 0xf0;
		} else {
			wav_samples[0] = 0xec;
		}
		if (j < len - 1 && (carrier_data->data[j] & 1) == (carrier_data->data[j+1] & 1)) {
			wav_samples[9] = 0xf0;
		} else {
			wav_samples[9] = 0xec;
		}
		for(int i=0; i<mag; i++) {
			wave_data->add((carrier_data->data[j] & 1) ? wav_samples[i] : 256 - wav_samples[i]);
		}
	}
	// TvO[gϊ
	pos = (int)conv.OutConvSampleData(wave_data->data, 48000, 8, 1, wave_data->w_pos
						, buffer, outwav.GetSampleRate(), outwav.GetSampleBits(), DATA_BUFFER_SIZE, 0);

	return pos;
}

/// @brief t@Cf[^ǂ݃VAf[^1bitɕϊďo͂
/// @param[in]  dir f[^is 0:Đ / 1: / -1:߂
/// @param[out] s_data  VAf[^ 1-2bit
/// @param[out] s_datalen VAf[^
/// @param[out] w_onedata NULL or wavef[^
/// @param[out] c_onedata NULL or VAf[^1-2rbg̔g
/// @return  / f[^I: 0
int PARSEWAV::GetData(int dir, char *s_data, int &s_datalen, char *w_onedata, int *w_onelen, char *c_onedata, int *c_onelen)
{
	int retry = 1;

	if (dir >= 0 && sample_pos >= sample_num) {
		if (w_onelen) *w_onelen = 0;
		if (c_onelen) *c_onelen = 0;
		return 0;
	} else if (dir < 0 && sample_pos == 0) {
		if (w_onelen) *w_onelen = 0;
		if (c_onelen) *c_onelen = 0;
		return 0;
	}

	do {
		decode_phase1(param.GetFskSpeed(), start_phase, param.GetCorrectType(), param.GetReverseWave(), w_onedata, w_onelen);
		if (dir == 0) decode_phase2(s_data, c_onedata, c_onelen);
		retry--;
	} while (s_data[0] == '?' && retry > 0);

	if (s_data[0] == '?') {
		strcpy(s_data, "1");
	}
	s_datalen = (int)strlen(s_data);

	return inwav_dirpos;
}

/// @brief VAf[^1bitϊăobt@ɏo͂
///
/// @param[in] data ̓f[^
/// @param[out] buffer ϊ̃f[^
/// @param[out] c_len  c_dataɕϊ̒
/// @return bufferɏ񂾃TCY
int PARSEWAV::SetData(uint8_t data, int &data_len, uint8_t *buffer, int *c_len)
{
	int len = 0;
	if (infile_type == 1) {
		// l3cf[^ɕϊ
		if (data_len >= 88 && (data & 1) == 0) {
			// cr + lf
			buffer[0]='\r';
			buffer[1]='\n';
			buffer += 2;
			len += 2;
			data_len = 0;
		}
		len += encode_phase2(data, buffer, c_len);
	} else {
		// waveTvf[^ɕϊ
		encode_phase1(data, buffer, c_len);
		len += wave_data->w_pos;
	}
	sample_pos += len;
	if (sample_num < sample_pos) {
		sample_num = sample_pos;
	}
	return len;
}

/// o͗p̃tH[}bgp[^ݒ
///
///
///
///
void PARSEWAV::SetFormatForSound(int rate, int bits, int ch)
{
	sndwav.Init(rate, bits, ch);
}
